@@ -1,7 +1,7 @@
 package App::BencherUtils;
 
-our $DATE = '2016-05-24'; # DATE
-our $VERSION = '0.12'; # VERSION
+our $DATE = '2016-06-23'; # DATE
+our $VERSION = '0.13'; # VERSION
 
 use 5.010001;
 use strict;
@@ -11,6 +11,7 @@ use Log::Any::IfLOG '$log';
 use Data::Clean::JSON;
 use Function::Fallback::CoreOrPP qw(clone);
 use Perinci::Sub::Util qw(err);
+use PerlX::Maybe;
 use POSIX qw(strftime);
 
 our %SPEC;
@@ -50,6 +51,8 @@ sub _json {
         require JSON::MaybeXS;
         my $json = JSON::MaybeXS->new;
         $json->convert_blessed(1);
+        $json->allow_nonref(1);
+        $json->canonical(1);
     };
     $json;
 }
@@ -238,7 +241,7 @@ sub list_bencher_results {
             "%s.%s.%s",
             $row->{scenario},
             $row->{module_startup} ? 0:1,
-            $row->{cpu},
+            $row->{cpu} // '', # when bencher is run --no-return-meta, there's no cpu information
         );
 
         if ($args{query} && @{ $args{query} }) {
@@ -301,6 +304,73 @@ sub list_bencher_results {
     }
 }
 
+$SPEC{cleanup_old_bencher_results} = {
+    v => 1.1,
+    summary => 'Delete old results',
+    description => <<'_',
+
+By default it will only keep 1 latest result for each scenario for the same CPU
+and the same module versions.
+
+You can use `--dry-run` first to see which files would be deleted without
+actually deleting them.
+
+_
+    args => {
+        %args_common,
+        %args_common_query,
+    },
+    features => {
+        dry_run => 1,
+    },
+};
+sub cleanup_old_bencher_results {
+    require File::Slurper;
+
+    my %args = @_;
+    my $res = list_bencher_results(
+        detail           => 1,
+        maybe result_dir => $args{result_dir},
+        maybe query      => $args{query},
+    );
+    return $res if $res->[0] != 200;
+
+    my @scenarios;
+    for my $scenario (@{ $res->[2] }) {
+        $scenario->{result} = _json->decode(File::Slurper::read_text(
+            "$args{result_dir}/$scenario->{filename}"));
+        push @scenarios, $scenario;
+    }
+
+    my %filenames; # key = scenario|cpu|module_startup(0|1)|json(module_versions), value = [filename, ...]
+    for my $scenario (@scenarios) {
+        my $key = join(
+            "|",
+            $scenario->{scenario},
+            $scenario->{cpu} // '',
+            $scenario->{module_startup} ? 1:0,
+            _encode_json($scenario->{result}[3]{'func.module_versions'}),
+        );
+        #$log->tracef("key = %s", $key);
+        push @{$filenames{$key}}, $scenario->{filename};
+    }
+
+    for my $key (sort keys %filenames) {
+        my $val = $filenames{$key};
+        next unless @$val > 1;
+        $val = [sort @$val];
+        for my $f (@{$val}[0..$#{$val}-1]) {
+            if ($args{-dry_run}) {
+                $log->warnf("[DRY-RUN] Deleting %s ...", $f);
+            } else {
+                $log->warnf("Deleting %s ...", $f);
+                unlink "$args{result_dir}/$f";
+            }
+        }
+    }
+    [200];
+}
+
 $SPEC{list_bencher_scenario_modules} = {
     v => 1.1,
     summary => 'List Bencher scenario modules',
@@ -342,7 +412,7 @@ App::BencherUtils - Utilities related to bencher
 
 =head1 VERSION
 
-This document describes version 0.12 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-05-24.
+This document describes version 0.13 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-06-23.
 
 =head1 SYNOPSIS
 
@@ -352,6 +422,8 @@ This distribution includes several utilities:
 
 =over
 
+=item * L<cleanup-old-bencher-results>
+
 =item * L<list-bencher-results>
 
 =item * L<list-bencher-scenario-modules>
@@ -361,6 +433,57 @@ This distribution includes several utilities:
 =head1 FUNCTIONS
 
 
+=head2 cleanup_old_bencher_results(%args) -> [status, msg, result, meta]
+
+Delete old results.
+
+By default it will only keep 1 latest result for each scenario for the same CPU
+and the same module versions.
+
+You can use C<--dry-run> first to see which files would be deleted without
+actually deleting them.
+
+This function is not exported.
+
+This function supports dry-run operation.
+
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<detail> => I<bool>
+
+=item * B<query> => I<array[str]>
+
+=item * B<result_dir>* => I<str>
+
+Directory to store results files in.
+
+=back
+
+Special arguments:
+
+=over 4
+
+=item * B<-dry_run> => I<bool>
+
+Pass -dry_run=>1 to enable simulation mode.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 list_bencher_results(%args) -> [status, msg, result, meta]
 
 List results in results directory.
